/*---------------------------------------------------------------------------
  Phase Field Model of crystal growth.

   $B&S&E(B^2 $B"_(Bt P = $B&E(B^2 $B"$(BP+P*(1-P)*(P-0.5+m(T,$B&H(B)+$B&D(B*RANSU)
   $B"_(Bt T = $B"$(BT+K$B"_(Bt P
   m(T,$B&H(B) = $B&A(B f($B&H(B) atan($B&B(B(1-T))/$B&P(B
   (cos$B&H(B,sin$B&H(B)=$B"`(BP/|$B"`(BP|

---------------------------------------------------------------------------*/

/*********** include **************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>

#define PI          M_PI
#define Random      (drand48()-0.5)
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))


/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double          t;
double          **P,**T;
double          DT,DX;
double          L,alpha,beta,tau,K,e,sigma;
double          delta,r_start;
int             Wimax,Wjmax,Dispstep=10;

void            initial(int);
inline double   M(double x,double theta)
{    double co=cos(3.0*theta);
     return alpha*(1.0-sigma*co*co)*atan(beta*(1-x))/PI;
}

void            boundary(void);

/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Control_Window  CW;
Graphic_Window  PW,TW;
int             PWsize;
void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_PhaseField.dvi",0);}  }


/********** main ***********/

int main(int argc,char **argv)
{
   if(argc>1) L=atof(argv[1]);     else L=3;
   if(argc>2) DT=atof(argv[2]);    else DT=0.0001;
   if(argc>3) DX=atof(argv[3]);    else DX=0.03;
   if(argc>4) K=atof(argv[4]);     else K=1.50;      
   if(argc>5) alpha=atof(argv[5]); else alpha=0.9;
   if(argc>6) beta=atof(argv[6]);  else beta=10.0;   
   if(argc>7) tau=atof(argv[7]);   else tau=3.0;
   if(argc>8) e=atof(argv[8]);     else e=0.01;
   if(argc>9) delta=atof(argv[9]); else delta=0.2;
   if(argc>10) sigma=atof(argv[9]); else sigma=0;   

   r_start=0.15;

   fprintf(stderr,
     "usage%% CGL_1d [L] [DT] [DX] [K] [alpha] [beta] [tau] [e] [delta] [sigma]\n"
     " current systemsize: L=%g\n"
     " current parameter : K=%g alpha=%g beta=%g tau=%g e=%g delta=%g sigma=%g\n"
     " space and time meshsize: DX=%g DT=%g\n"
     ,L,K,alpha,beta,tau,e,delta,sigma,DX,DT); 

   Wimax=Wjmax=(int)(L/DX);  
   array(P,Wimax+1,Wjmax+1); 
   array(T,Wimax+1,Wjmax+1);

   double   **T2,**P2,**P3;
   array(P2,Wimax+1,Wjmax+1); array(P3,Wimax+1,Wjmax+1);
   array(T2,Wimax+1,Wjmax+1);

/*----- initialize ----------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;

   CW<< t << initial<<"t"<<"initial"<<PARAMETER
     << alpha << beta << K << tau << e << sigma
     <<"alpha"<<"beta"<<"K"<<"tau"<<"e"<<"sigma";
   
   PW.geometry(Wimax+1,Wjmax+1,2)<<"P(order parameter)";
   TW.geometry(Wimax+1,Wjmax+1,2)<<"T(temperature)";

   initialize();  

   int i,j,ts;

/*----- main routine --------*/

   double   nl,dP,dT,tee,txx,txxp,theta,dPx,dPy;
   
   initial(0);   
   while(1){
      checkevent();

      tee=DT/(e*e*tau);
      txx=DT/(DX*DX);
      txxp=txx/tau;

/*----- P local map -----*/

      for(ts=0; ts<Dispstep;ts++){
        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
	     dPx=P[i+1][j]-P[i-1][j];
	     dPy=P[i][j+1]-P[i][j-1];
	     theta=atan2(dPx,dPy);
             nl=P[i][j]*(1-P[i][j])
	       *(P[i][j]-0.5+M(T[i][j],theta)+delta*Random);
             P2[i][j]=P[i][j]+tee*nl;
          }

/*----- P & T diffusion term -----*/

        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
             dP=P[i-1][j]+P[i+1][j]+P[i][j-1]+P[i][j+1]-4.0*P[i][j];
             P3[i][j]=P2[i][j]+txxp*dP;
             dT=T[i-1][j]+T[i+1][j]+T[i][j-1]+T[i][j+1]-4.0*T[i][j];
             T2[i][j]=T[i][j]+txx*dT;
          }

/*----- T local map -----*/
        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
             T[i][j]=T2[i][j]+K*(P3[i][j]-P[i][j]);
             if( T[i][j] < 0 ) T[i][j]=0.0;
             P[i][j]=P3[i][j];
          }
	
        boundary();
      }
      
      t+=DT*Dispstep;
      PW.draw_2D(P);
      TW.draw_2D(T);
   }
}


/**********************  $B=i4|>r7o(B *************************/

void initial(int)
{
    for(int j=0; j<Wjmax+1; j++)
        for(int i=0; i<Wimax+1; i++){ P[i][j] = 0; T[i][j] = 0; }

    for(int j=1; j<Wjmax; j++)
        for(int i=1; i<Wimax; i++)
            if(((i-1)*(i-1)+(j-1)*(j-1))<(r_start/DX*r_start/DX)){
                T[i][j] = 1.0; P[i][j] = 1.0;
            }

    boundary();
    t=0;
}


/**********************  $B6-3&>r7o(B *************************/

void boundary(void)
{
    for(int j=1;j<Wjmax;j++){
        T[0][j]=T[1][j]; P[0][j]=P[1][j];
        T[Wimax][j]=0;   P[Wimax][j]=0;
    }
    
    for(int i=1;i<Wimax;i++){
        T[i][0]=T[i][1]; P[i][0]=P[i][1];
        T[i][Wjmax]=0;   P[i][Wjmax]=0;
    }
}







    


