/*==========================================================================
    A model of collective motion in swarms
      proposed by A. S. Mikhailov and D. H. Zanette
      (Phys.Rev. E Vol.60 pp.4571 1999)

      dxdt=v
      dvdt=(1-v*v)*v+a*(<x>-x)+$B&N(B(t)+F(x)

      <x> = average of x;
      <$B&N(B(t)$B&N(B(t')>=s*s*$B&D(B(t-t') : white noise

===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
#include <XRS/Plane.h>

#define  Random   drand48()-0.5
#define  Noise    ((drand48()>0.5)?1.0:-1.0)
#define  Color    6


/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double    a,b,s;                          // $B%Q%i%a!<%?(B
double    *x,*v;                          // $B>l(B
double    xc,vc;                          // $B=i4|>r7o$NCf?4:BI8(B
int       N;                              // $B%7%9%F%`%5%$%:(B
double    t;                              // $B;~4V(B

void            initial(int);


/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/

Control_Window  CW;
Plane_Window    PW;
int             PWsize=200;
int             Initial=0,Trace=1;
double          Amp=0.2;

void            draw();
void            trace(int){ Trace=!Trace; }
void            explain(int)
{  if(!fork()){ execlp("xdvi","xdvi","-s","8","README_swarms.dvi",0);}  }

double          F(double x){ double y=b*x; return -y*y*y; }


int main(int argc,char **argv)
{
   int       i;   
   double    DT;
   double    X,t0,t1,t2;

   /*--------------- initialize ---------------------*/
   
   if(argc>1) N =atoi(argv[1]);  else N=500;
   if(argc>2) DT=atof(argv[2]);  else DT=0.02;
   if(argc>3) a=atof(argv[3]);   else a=1.0;
   if(argc>4) s=atof(argv[4]);   else s=0;
   if(argc>5) b=atof(argv[5]);   else b=0.2;

   fprintf(stderr,"usage%% swarm [N] [DT] [a] [s] [b]\n"
          " current systemsize: N=%d\n"
          " current parameter : a=%g s=%g b=%g\n"
	  " space and time meshsize: DT=%g\n"
   	  ,N,a,s,b,DT); 

   PWsize=200;
   x  = new double [N];   
   v  = new double [N];   

   /*---------------- setup window -------------------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;
   
   CW<<t<<"t"<<xc<<"xc"<<vc<<"vc"<<trace<<"Trace"<<PARAMETER
     << a << s << b << Amp 
     <<"a"<<"s"<<"b"<<"Amp";
   
   PW.geometry(PWsize,PWsize)<<"x-v plane"<<initial;
   
   initialize();
   
   /*--------------- main routine -------------------*/

   initial(0);
   while(1){
      checkevent();

      t0=DT;
      t1=a*DT;
      t2=s*sqrt(DT*2.0);

      X=0; for(i=0; i<N; i++) X+=x[i];  X/=N;
      for(i=0; i<N; i++){
           x[i]+=DT*v[i];
           v[i]+=t0*(1.0-v[i]*v[i])*v[i]+t1*(X-x[i])+t0*F(x[i])+t2*Noise;
      }

      t+=DT;
      draw();
   }
}


/********************** $B=q$/(B *************************/

#define PWinI(x) ((int)(PWsize*0.5*(Amp*(x)+1.0)))
#define PWinJ(x) ((int)(PWsize*0.5*(-Amp*(x)+1.0)))		  
#define PWinX(i) ((double)((2.0*(i)/PWsize-1.0)/Amp))
#define PWinY(j) ((double)(-(2.0*(j)/PWsize-1.0)/Amp))		  

		  
void draw(void)
{
    /*======= complex plane snapshot =========*/

    if(Initial){
	PW.clear();
	PW.draw_line(1,PWinI(0),0,PWinI(0),PWsize);
	PW.draw_line(1,0,PWinJ(0),PWsize,PWinJ(0));	
    }
    
    if(Trace) PW.clear(0);
    for(int i=0; i<N; i++){
       PW.draw_point(0,PWinI(x[i]),PWinJ(v[i]));
    }

    PW.clear(2);
    PW.draw_circle(2,PWinI(x[0]),PWinJ(v[0]),4);

    if(Initial) Initial=0; 
}



/**********************  $B=i4|>r7o(B *************************/

void initial(int n)
{ 
   if(n==0) xc=vc=0;
   xc=PWinX(PW.ev.xbutton.x);
   vc=PWinY(PW.ev.xbutton.y);

   for(int i=0; i<N; i++){ x[i]=Random+xc; v[i]=Random+vc;}

   t=0; Initial=1;
}





